from django_anchor_modeling import constants
from django_anchor_modeling.exceptions import SentinelUserDoesNotExist


def get_knot_choice_value(instance: object):
    """
    Given any instance of a model, return the knot choice value
    format is

    APP__CLASS
    """
    app_label = instance._meta.app_label
    class_name = instance.__class__.__name__

    return f"{app_label}__{class_name}"


# utils.py or a similar module
from django.conf import settings
from django.contrib.auth import get_user_model

User = get_user_model()

SENTINEL_NULL_USER_ID = getattr(
    settings, "SENTINEL_NULL_USER_ID", constants.SENTINEL_NULL_USER_ID
)


def get_sentinel_user():
    sentinel_user, _ = User.objects.get_or_create(pk=SENTINEL_NULL_USER_ID)
    return sentinel_user


def get_sentinel_user_id():
    if User.objects.filter(pk=SENTINEL_NULL_USER_ID).exists():
        return SENTINEL_NULL_USER_ID
    else:
        raise SentinelUserDoesNotExist()


class DynamicAttributeDescriptor:
    def __init__(self, field_name):
        self.field_name = field_name

    def __get__(self, instance, owner):
        if instance is None:
            return self  # Access through class, not instance

        try:
            attribute = getattr(instance, self.field_name)
            return attribute.value
        except AttributeError as e:
            raise AttributeError from e



# Generated by Django 4.2.6 on 2024-03-24 15:52

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_anchor_modeling.utils


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('django_anchor_modeling', '0002_sentinel_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='transaction',
            name='created_by',
            field=models.ForeignKey(db_constraint=False, default=django_anchor_modeling.utils.get_sentinel_user_id, editable=False, on_delete=django.db.models.deletion.DO_NOTHING, related_name='created_%(app_label)s_%(class)s', related_query_name='query_created_%(app_label)s_%(class)ss', to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 4.2.6 on 2023-11-13 00:25

from django.db import migrations, models

import django_anchor_modeling.fields


class Migration(migrations.Migration):
    dependencies = [
        ("dataviewer", "0003_remove_businesstodatafieldmap_metadata"),
    ]

    operations = [
        migrations.CreateModel(
            name="BusinessToQueryMap",
            fields=[
                ("created", models.DateTimeField(auto_now_add=True)),
                (
                    "id",
                    django_anchor_modeling.fields.BusinessIdentifierField(
                        max_length=255, primary_key=True, serialize=False
                    ),
                ),
                ("description", models.TextField()),
                ("main_model_class", models.CharField(max_length=255)),
                ("select_related", models.JSONField(default=list)),
                ("prefetch_related", models.JSONField(default=list)),
                ("only", models.JSONField(default=list)),
                ("field_processors", models.JSONField(default=dict)),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



# Generated by Django 4.2.6 on 2023-10-28 23:13

from django.conf import settings
from django.db import migrations

SENTINEL_NULL_USER_ID = getattr(settings, "SENTINEL_NULL_USER_ID", -1)


def add_sentinel_user(apps, schema_editor):
    UserModel = apps.get_model(settings.AUTH_USER_MODEL)
    UserModel.objects.get_or_create(pk=SENTINEL_NULL_USER_ID)


def remove_sentinel_user(apps, schema_editor):
    UserModel = apps.get_model(settings.AUTH_USER_MODEL)
    UserModel.objects.filter(pk=SENTINEL_NULL_USER_ID).delete()


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        (
            "auth",
            "0012_alter_user_first_name_max_length",
        ),  # Ensure this dependency matches your current Django version's auth migration
        ("django_anchor_modeling", "0001_initial"),
    ]

    operations = [
        migrations.RunPython(add_sentinel_user, remove_sentinel_user),
    ]
